# -*- coding: utf-8 -*-
"""
Created on Tue Dec  6 09:32:53 2016

@author: eric
"""

# Insure compatibility.
from __future__ import absolute_import, division, print_function, unicode_literals

# Import utilities.
import tipi

# Import modules using the same conventions as SciPy:
import numpy as np              # to deal with numerical arrays
import matplotlib as mpl        # for graphics
import matplotlib.pyplot as plt # for plotting
from astropy.io import fits
plt.ion() # or leave this out and run with ipython --pylab

# Other modules
import time

dat = tipi.fits_read('data/saturn/saturn.fits')
psf = tipi.fits_read('data/saturn/saturn_psf.fits')
mtf = tipi.fft(tipi.ifftshift(tipi.zeropad(psf, dat.shape)))

plt.figure(1)
plt.clf()
plt.imshow(dat, cmap='viridis', vmin=0)
plt.colorbar()
plt.title('Image originale')

def rundemo(bad=None, mu=0.0, maxiter=100, grtol=1e-8, gatol=0, fig=2):
    wgt = np.ones(dat.shape)
    wgt[dat <= np.min(dat)] = 0
    if bad is not None:
        wgt[np.random.rand(wgt.shape[0],wgt.shape[1]) < bad] = 0
    if np.max(wgt) <= 0:
        print("Trop de mauvaises données !")
        return
    if fig is not None:
        plt.figure(fig)
        plt.subplot(121)
        plt.cla()
        msk = np.ones(dat.shape)
        msk[wgt <= 0] = 0
        plt.imshow(dat*msk, vmin=0, cmap='viridis', interpolation='nearest')
        nbads = np.sum(msk)
        if nbads > 1:
            pc = round(1e2 - 1e2*nbads/msk.size, 1)
            plt.title('Données disponibles\n({}% invalides)'.format(pc))
        else:
            plt.title('Image brute')
        plt.subplot(122)
    def view(img, k, rho):
        ax = plt.gca()
        ax.cla()
        ax.imshow(img, vmin=0, cmap='viridis', interpolation='nearest')
        #plt.colorbar()
        ax.set_title('Image reconstruite\n({0} iteration{1})'.format(k, 's' if k > 0 else ''))
        plt.gcf().canvas.draw()
        time.sleep(0.001)
    def W(v): return wgt*v
    def H(v) : return np.real(tipi.ifft(mtf*tipi.fft(v)))
    def Ht(v) : return np.real(tipi.ifft(np.conj(mtf)*tipi.fft(v)))
    if mu > 0:
        def A(x): return Ht(W(H(x))) +  mu*tipi.DtD(x)
    else:
        def A(x): return Ht(W(H(x)))
    b = Ht(W(dat))
    img = tipi.conjgrad(A, b, x=None, maxiter=maxiter, gatol=gatol,
                        grtol=grtol, quiet=False, viewer=view)
    return img


